import inspect
import os
from pathlib import Path


def path_from_caller(module):
    if module and hasattr(module, '__file__'):
        directory = Path(module.__file__).resolve().parent
        return directory
    else:
        return None


def path_from_cwd():
    return Path.cwd()


def path_from_project():
    # 获取项目根目录 环境变量 PROJECT_ROOT
    path = os.environ.get('PROJECT_ROOT', None)
    if path is not None:
        return Path(path)
    return None


def to_absolute_path(path):
    frame = inspect.stack()[1]
    module = inspect.getmodule(frame[0])
    caller_path = path_from_caller(module)
    cwd_path = path_from_cwd()
    project_path = path_from_project()
    # print(caller_path)
    # print(cwd_path)
    # print(project_path)
    if caller_path is not None:
        abs_path = caller_path / path
        if abs_path.exists():
            return abs_path

    if cwd_path is not None:
        abs_path = cwd_path / path
        if abs_path.exists():
            return abs_path

    if project_path is not None:
        abs_path = project_path / path
        if abs_path.exists():
            return abs_path

    return None


def to_absolute_path_str(path):
    frame = inspect.stack()[1]
    module = inspect.getmodule(frame[0])
    caller_path = path_from_caller(module)
    cwd_path = path_from_cwd()
    project_path = path_from_project()
    # print(caller_path)
    # print(cwd_path)
    # print(project_path)
    if caller_path is not None:
        abs_path = caller_path / path
        if abs_path.exists():
            return abs_path.as_posix()

    if cwd_path is not None:
        abs_path = cwd_path / path
        if abs_path.exists():
            return abs_path.as_posix()

    if project_path is not None:
        abs_path = project_path / path
        if abs_path.exists():
            return abs_path.as_posix()

    return None


import os


def list_files(paths, suffixes=None, absolute=True):
    """
    遍历指定路径，返回一个带路径前缀的展平文件列表。

    参数:
        paths (str or list): 要遍历的路径，支持字符串或路径列表。
        suffixes (list): 允许的文件后缀名列表，如 [".txt", ".csv"]，默认 None 表示不限制。
        absolute (bool): 是否返回绝对路径，默认 True。

    返回:
        list: 带路径前缀的展平文件列表。
    """
    result = []

    if isinstance(paths, str):
        paths = [paths]

    for path in paths:
        abs_base = os.path.abspath(path)

        for root, _, files in os.walk(path):
            for file in files:
                ext = os.path.splitext(file)[1].lower()
                if suffixes is None or ext in suffixes:
                    full_path = os.path.join(root, file)
                    if absolute:
                        result.append(full_path)
                    else:
                        rel_path = os.path.relpath(full_path, start=abs_base)
                        result.append((abs_base, rel_path))
    return result


def list_files_with_follow(fds):
    if isinstance(fds, dict):
        fds = [fds]
    files_dict = []
    for fd in fds:
        path, *suffixes = fd['main']
        main_file_list = list_files(path, suffixes=suffixes, absolute=False)
        for _, main_file in main_file_list:
            file_dict = [os.path.join(path, main_file)]
            for follow in fd.get('follows', []):
                follow_dir = follow[0]
                follow_file_path_add = None
                for suffix in follow[1:]:
                    main_file_without_suffix = os.path.join(os.path.dirname(main_file), os.path.splitext(os.path.basename(main_file))[0])
                    follow_file_path = os.path.join(follow_dir, f'{main_file_without_suffix}{suffix}')
                    if os.path.exists(follow_file_path):
                        follow_file_path_add = follow_file_path
                        break
                file_dict.append(follow_file_path_add)
            files_dict.append(file_dict)
    return files_dict


if __name__ == '__main__':
    # file_paths = list_files([r'S:\videos\tennis_train_data\detect\train',
    #                          r'D:\Code\aigc-image-ai\service\video_anysis\v2_20250704\converted_photos'],
    #                         suffixes=['.jpg', '.png'], absolute=True)

    fds = [{'main': [r'S:\videos\tennis_train_data\detect\train', '.jpg', '.png'],
            'follows': [[r'S:\videos\tennis_train_data\detect\train', '.txt']]}]
    data = list_files_with_follow(fds)
    print(data)



import numpy as np


def xywh2xyxy(xywh):
    if len(xywh) == 0:
        return xywh

    if isinstance(xywh, dict):
        return {'x1': xywh[0], 'y1': xywh[1], 'x2': xywh[0] + xywh[2], 'y2': xywh[1] + xywh[3]}
    data_list = []

    if isinstance(xywh[0], list):
        for _xywh in xywh:
            data_list.append([_xywh[0], _xywh[1], _xywh[0] + _xywh[2], _xywh[1] + _xywh[3]])
    elif isinstance(xywh[0], np.ndarray):
        for _xywh in xywh:
            data_list.append(np.array([_xywh[0], _xywh[1], _xywh[0] + _xywh[2], _xywh[1] + _xywh[3]]))
    elif isinstance(xywh[0], tuple):
        for _xywh in xywh:
            data_list.append((_xywh[0], _xywh[1], _xywh[0] + _xywh[2], _xywh[1] + _xywh[3]))
    elif isinstance(xywh[0], dict):
        for _xywh in xywh:
            data_list.append({'x1': _xywh[0], 'y1': _xywh[1], 'x2': _xywh[0] + _xywh[2], 'y2': _xywh[1] + _xywh[3]})
    else:
        data_list = [xywh[0], xywh[1], xywh[0] + xywh[2], xywh[1] + xywh[3]]
    if isinstance(xywh, list):
        return data_list
    elif isinstance(xywh, np.ndarray):
        return np.array(data_list)
    elif isinstance(xywh, tuple):
        return tuple(data_list)


def xywh2xyxy_list(xywh):
    xyxy = xywh2xyxy(xywh)
    data_list = []
    for _xyxy_ in xyxy:
        if isinstance(_xyxy_, np.ndarray) or isinstance(_xyxy_, list) or isinstance(_xyxy_, tuple):
            data_ele = []
            for xy2 in _xyxy_:
                if 'int' in str(type(xy2)):
                    data_ele.append(int(xy2))
                elif 'float' in str(type(xy2)):
                    data_ele.append(float(xy2))
            data_list.append(data_ele)
        else:
            if 'int' in str(type(_xyxy_)):
                data_list.append(int(_xyxy_))
            elif 'float' in str(type(_xyxy_)):
                data_list.append(float(_xyxy_))
    return data_list


def xywh2xyxy_np(xywh):
    return np.array(xywh2xyxy_list(xywh))


def xyxy2xywh(xyxy):
    if len(xyxy) == 0:
        return xyxy

    if isinstance(xyxy, dict):
        return {'x': xyxy[0], 'y': xyxy[1], 'w': xyxy[2] - xyxy[0], 'h': xyxy[3] - xyxy[1]}
    data_list = []

    if isinstance(xyxy[0], list):
        for _xyxy in xyxy:
            data_list.append([_xyxy[0], _xyxy[1], _xyxy[2] - _xyxy[0], _xyxy[3] - _xyxy[1]])
    elif isinstance(xyxy[0], np.ndarray):
        for _xyxy in xyxy:
            data_list.append(np.array([_xyxy[0], _xyxy[1], _xyxy[2] - _xyxy[0], _xyxy[3] - _xyxy[1]]))
    elif isinstance(xyxy[0], tuple):
        for _xyxy in xyxy:
            data_list.append((_xyxy[0], _xyxy[1], _xyxy[2] - _xyxy[0], _xyxy[3] - _xyxy[1]))
    elif isinstance(xyxy[0], dict):
        for _xyxy in xyxy:
            data_list.append({'x': _xyxy[0], 'y': _xyxy[1], 'w': _xyxy[2] - _xyxy[0], 'h': _xyxy[3] - _xyxy[1]})
    else:
        data_list = [xyxy[0], xyxy[1], xyxy[2] - xyxy[0], xyxy[3] - xyxy[1]]
    if isinstance(xyxy, list):
        return data_list
    elif isinstance(xyxy, np.ndarray):
        return np.array(data_list)
    elif isinstance(xyxy, tuple):
        return tuple(data_list)


def xyxy2xywh_list(xyxy):
    xywh = xyxy2xywh(xyxy)
    data_list = []
    for _xywh_ in xywh:
        if isinstance(_xywh_, np.ndarray) or isinstance(_xywh_, list) or isinstance(_xywh_, tuple):
            data_ele = []
            for xy2 in _xywh_:
                if 'int' in str(type(xy2)):
                    data_ele.append(int(xy2))
                elif 'float' in str(type(xy2)):
                    data_ele.append(float(xy2))
            data_list.append(data_ele)
        else:
            if 'int' in str(type(_xywh_)):
                data_list.append(int(_xywh_))
            elif 'float' in str(type(_xywh_)):
                data_list.append(float(_xywh_))
    return data_list


def xyxy2xywh_np(xyxy):
    return np.array(xyxy2xywh_list(xyxy))


def bbox_iou(box1, box2):
    b1_x1, b1_y1, b1_x2, b1_y2 = box1
    b2_x1, b2_y1, b2_x2, b2_y2 = box2

    inter_x1 = max(b1_x1, b2_x1)
    inter_y1 = max(b1_y1, b2_y1)
    inter_x2 = min(b1_x2, b2_x2)
    inter_y2 = min(b1_y2, b2_y2)

    if inter_x2 < inter_x1 or inter_y2 < inter_y1:
        return 0.0

    inter_area = (inter_x2 - inter_x1) * (inter_y2 - inter_y1)

    area_box1 = (b1_x2 - b1_x1) * (b1_y2 - b1_y1)
    area_box2 = (b2_x2 - b2_x1) * (b2_y2 - b2_y1)

    union_area = area_box1 + area_box2 - inter_area

    iou = inter_area / union_area

    return iou



import logging
import threading

from confluent_kafka import Producer, Consumer

from .channel import MqSender, MqReceiver

logger = logging.getLogger(__name__)


class KafkaSender(MqSender):
    def __init__(self, bootstrap_servers, topic=None, security_protocol=None, sasl_mechanism=None,
                 sasl_plain_username=None, sasl_plain_password=None):
        super().__init__()
        self.topic = topic
        # logger.info(f'{(security_protocol, sasl_mechanism, sasl_plain_username, sasl_plain_password)}')
        if security_protocol and sasl_mechanism:
            self.producer = Producer({
                "bootstrap.servers": bootstrap_servers,
                'security.protocol': security_protocol,
                'sasl.mechanism': sasl_mechanism,
                'sasl.username': sasl_plain_username,
                'sasl.password': sasl_plain_password
            })
        else:
            self.producer = Producer({
                "bootstrap.servers": bootstrap_servers
            })

    def sending(self, feedback, topic=None, key=None):
        super().sending(feedback=feedback)
        topic = self.topic if topic is None else topic
        self._produce(feedback, topic, key)

    def _produce(self, message, topic=None, key=None):

        def acked(err, msg):
            if err is not None:
                logger.error('Failed to deliver message:{}'.format(err))
            else:
                logger.debug(f'message sent successfully through Topic {topic} with key {key}')

        logger.debug(f'message (type is {type(message)}) sent through Topic {topic} with key {key}: {message}')
        # Asynchronous by default ( 默认是异步发送 )
        self.producer.produce(
            topic=topic,
            key=key,
            value=message,
            callback=acked
        )
        self.producer.flush(timeout=30)


class KafkaReceiver(MqReceiver):

    def __init__(self, bootstrap_servers, topics, group_id, security_protocol=None, sasl_mechanism=None,
                 sasl_plain_username=None, sasl_plain_password=None):
        super().__init__()
        # logger.info(f'{(security_protocol, sasl_mechanism, sasl_plain_username, sasl_plain_password)}')
        if security_protocol and sasl_mechanism:
            self.consumer = Consumer({
                "bootstrap.servers": bootstrap_servers,
                'security.protocol': security_protocol,
                'sasl.mechanism': sasl_mechanism,
                'sasl.username': sasl_plain_username,
                'sasl.password': sasl_plain_password,
                'group.id': group_id,
                'max.poll.interval.ms': 3600000,
                'auto.offset.reset': 'earliest'
            })
        else:
            self.consumer = Consumer({
                "bootstrap.servers": bootstrap_servers,
                'group.id': group_id,
                'max.poll.interval.ms': 3600000,
                'auto.offset.reset': 'earliest'
            })
        self.consumer.subscribe(topics=topics)

    def _start_consuming(self):
        while True:
            try:
                msg_dict = self.consumer.poll(0.1)
                if msg_dict is None:
                    continue
                if msg_dict.error():
                    logger.error("Consumer error:{}".format(msg_dict.error()))
                    continue
                message = msg_dict.value().decode('utf-8')
                logger.info(
                    f'Kafka topic partition=({msg_dict.topic()}, {msg_dict.partition()}), offset={msg_dict.offset()},\
timestamp={msg_dict.timestamp()}, headers={msg_dict.headers()}, key={msg_dict.key()}, \
value={"--too long--" if len(message) > 500 else message}')

                super()._receive(message=message)
            except:
                logger.exception('')

    def start_receiving(self):
        super().start_receiving()
        # 新建 thread 启动
        threading.Thread(target=self._start_consuming).start()
